"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LineComposer = void 0;
const trivia_1 = require("./trivia");
class LineComposer {
    constructor(remainingWidth, theme, renderWhitespace = true) {
        this.remainingWidth = remainingWidth;
        this.theme = theme;
        this.renderWhitespace = renderWhitespace;
        this.parts = [];
    }
    append(part, brush = this.theme.foreground("editor.foreground")) {
        if (part.length === 0 || this.remainingWidth === 0) {
            // width limit reached. do nothing.
        }
        else if (part.length <= this.remainingWidth) {
            // whole part can fit.
            this.parts.push([part, brush]);
            this.remainingWidth -= part.length;
        }
        else {
            // only add the remaining characters and elepsis
            this.parts.push([part.substr(0, this.remainingWidth - trivia_1.ELEPSIS.length), brush]);
            this.parts.push([trivia_1.ELEPSIS, brush]);
            this.remainingWidth = 0;
        }
    }
    compose() {
        const whitespaceBrush = this.theme.foreground("editorWhitespace.foreground");
        const withColor = this.parts.map(([part, brush]) => part
            .split(/ /g)
            .map((atom) => atom
            .split(/\t/g)
            .map((p) => brush(p))
            .join(whitespaceBrush(this.renderWhitespace ? trivia_1.TAB_REPLACEMENT : "\t")))
            .join(whitespaceBrush(this.renderWhitespace ? trivia_1.WHITESPACE_REPLACEMENT : " ")));
        return withColor.join("") + " ".repeat(this.remainingWidth);
    }
}
exports.LineComposer = LineComposer;
//# sourceMappingURL=line-composer.js.map